package cc.fascinated.bat.event;

import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.model.DiscordMessage;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberLeaderboardToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberPlayerScoreToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberScoreToken;
import lombok.NonNull;
import net.dv8tion.jda.api.events.guild.member.GuildMemberJoinEvent;
import net.dv8tion.jda.api.events.guild.member.GuildMemberRemoveEvent;
import net.dv8tion.jda.api.events.guild.member.update.GuildMemberUpdateNicknameEvent;
import net.dv8tion.jda.api.events.interaction.ModalInteractionEvent;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import net.dv8tion.jda.api.events.interaction.component.StringSelectInteractionEvent;
import net.dv8tion.jda.api.events.message.MessageDeleteEvent;
import net.dv8tion.jda.api.events.message.MessageReceivedEvent;
import net.dv8tion.jda.api.events.message.MessageUpdateEvent;
import net.dv8tion.jda.api.events.user.update.UserUpdateGlobalNameEvent;

/**
 * @author Fascinated (fascinated7)
 */
public interface EventListener {
    /**
     * Called when a ScoreSaber score is received
     *
     * @param score       the score that was set
     * @param leaderboard the leaderboard that the score was set on
     * @param player      the player that set the score
     */
    default void onScoresaberScoreReceived(@NonNull ScoreSaberPlayerScoreToken score, @NonNull ScoreSaberLeaderboardToken leaderboard,
                                           @NonNull ScoreSaberScoreToken.LeaderboardPlayerInfo player) {
    }

    /**
     * Called when a user joins a guild
     *
     * @param guild the guild the user joined
     * @param user  the user that joined the guild
     */
    default void onGuildMemberJoin(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull GuildMemberJoinEvent event) {
    }

    /**
     * Called when a user leaves a guild
     *
     * @param guild the guild the user left
     * @param user  the user that left the guild
     */
    default void onGuildMemberLeave(@NonNull BatGuild guild, BatUser user, @NonNull GuildMemberRemoveEvent event) {
    }

    /**
     * Called when a user types a message
     *
     * @param guild the guild that the message was sent in
     * @param user  the user that sent the message
     */
    default void onGuildMessageReceive(@NonNull BatGuild guild, @NonNull BatUser user, @NonNull MessageReceivedEvent event) {
    }

    /**
     * Called when a user updates a message
     *
     * @param guild the guild that the message was updated in
     * @param user  the user that updated the message
     */
    default void onGuildMessageEdit(@NonNull BatGuild guild, @NonNull BatUser user, DiscordMessage oldMessage,
                                    @NonNull DiscordMessage newMessage, @NonNull MessageUpdateEvent event) {
    }

    /**
     * Called when a user deletes a message
     *
     * @param guild the guild that the message was deleted in
     * @param user  the user that deleted the message
     */
    default void onGuildMessageDelete(@NonNull BatGuild guild, BatUser user, DiscordMessage message, @NonNull MessageDeleteEvent event) {
    }

    /**
     * Called when a user selects a string
     *
     * @param guild the guild that the string was selected in
     * @param user  the user that selected the string
     */
    default void onStringSelectInteraction(BatGuild guild, @NonNull BatUser user, @NonNull StringSelectInteractionEvent event) {
    }

    /**
     * Called when a user interacts with a button
     *
     * @param guild the guild that the button was interacted with in
     * @param user  the user that interacted with the button
     */
    default void onButtonInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ButtonInteractionEvent event) {
    }

    /**
     * Called when a user interacts with a modal
     *
     * @param guild the guild that the modal was interacted with in
     * @param user  the user that interacted with the modal
     */
    default void onModalInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ModalInteractionEvent event) {
    }

    /**
     * Called when a user updates their global name
     *
     * @param user    the user that updated their global name
     * @param oldName the old global name
     * @param newName the new global name
     */
    default void onUserUpdateGlobalName(@NonNull BatUser user, String oldName, String newName, @NonNull UserUpdateGlobalNameEvent event) {
    }

    /**
     * Called when a user updates their nickname in a guild
     *
     * @param guild   the guild that the user updated their nickname in
     * @param user    the user that updated their nickname
     * @param oldName the old nickname
     * @param newName the new nickname
     */
    default void onGuildMemberUpdateNickname(@NonNull BatGuild guild, @NonNull BatUser user, String oldName, String newName, @NonNull GuildMemberUpdateNicknameEvent event) {
    }

    /**
     * Called when Spring is shutting down
     */
    default void onSpringShutdown() {
    }
}
