package cc.fascinated.bat.features.logging;

import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor @Getter
public enum LogType {
    /**
     * Message Events
     */
    MESSAGE_DELETE(LogCategory.MESSAGES, "Message Delete"),
    MESSAGE_EDIT(LogCategory.MESSAGES,"Message Edit"),

    /**
     * Member Events
     */
    MEMBER_JOIN(LogCategory.MEMBER, "Member Join"),
    MEMBER_LEAVE(LogCategory.MEMBER, "Member Leave"),
    MEMBER_NICKNAME_CHANGE(LogCategory.MEMBER, "Member Nickname Change");

    /**
     * The category of the log type
     */
    private final LogCategory category;

    /**
     * The name of the log type
     */
    private final String name;

    /**
     * Gets the log type by the name
     *
     * @param name - the name
     * @return the log type, or null if it doesn't exist
     */
    public static LogType getLogType(String name) {
        for (LogType logType : values()) {
            if (logType.getName().equalsIgnoreCase(name)) {
                return logType;
            }
        }
        return null;
    }

    /**
     * Gets the log types by the category
     *
     * @param category - the category
     * @return the log types
     */
    public static List<LogType> getLogTypesByCategory(String category) {
        List<LogType> logTypes = new ArrayList<>();
        for (LogType logType : values()) {
            if (logType.getCategory().getName().equalsIgnoreCase(category)) {
                logTypes.add(logType);
            }
        }
        return logTypes;
    }
}
